#' Generate a Summary of the data in the s2dv_cube
#'
#'This function prints the summary of an object of class \code{s2dv_cube},
#'loaded using \code{CST_Start}.
#'   
#' @author Theertha Kariyathan, \email{theertha.kariyathan@bsc.es}
#'
#' @param data_cube An \code{s2dv_cube} object containing:
#'   - `data`: N-dimensional array with named dimensions
#'   - `dims`: Dimensions, including `var` (variables).
#'   - `attrs`: Attributes such as `VarName` and `Metadata`.
#'   - `coords`: Named list with coordinates of dimensions.
#'   
#' @return A printed summary of the \code{s2dv_cube} object, including:
#'   - Months that have been loaded.
#'   - Range of the dates that have been loaded.
#'   - Object dimensions.
#'   - Basic statistical summary of the data.
#'   - Missing files.
#'   - Number of NAs per time dimension and latitude/longitude dimensions
#'   - Variables that have been loaded, along with their units
#'   
#' @details The function uses the metadata from the s2dv cube to extract 
#' variable names and units. It prints a detailed summary of all the variables loaded into the 
#' \code{s2dv_cube} object.
#'
#' @examples
#' # Example s2dv cube paths
#' repos <- "/esarchive/exp/ecmwf/system5_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc"
#' repos2 <- "/esarchive/exp/ecmwf/system4_m1/monthly_mean/$var$_f6h/$var$_$sdate$.nc"
#'
#' # Create data cube
#' data_cube <- CST_Start(dat = list(
#'                 list(name = 'system4_m1', path = repos2),
#'                 list(name = 'system5_m1', path = repos)),
#'                 var = c('tas', 'sfcWind'),
#'                 sdate = '20170101',
#'                 ensemble = indices(1),
#'                 time = indices(1),
#'                 lat = indices(1:5),
#'                 lon = indices(1:5),
#'                 synonims = list(lat = c('lat', 'latitude'),
#'                                 lon = c('lon', 'longitude')),
#'                 return_vars = list(time = 'sdate',
#'                                    longitude = 'dat',
#'                                    latitude = 'dat'),
#'                 metadata_dims = c('dat', 'var'),
#'                 retrieve = TRUE)
#'
#' # Generate summary
#' CST_summary(data_cube)
#'
#' @seealso \link[CSTools]{CST_start} for creating an s2dv cube object.
#' @importFrom StartR Start
#' @export
#' 
#' 
CST_summary <- function(data_cube) {
  # Get name, leadtime months and date range
  object_name <- deparse(substitute(data_cube))
  
  date_format <- "%b %d %Y"
  months <- unique(format(as.Date(data_cube$attrs$Dates), format = '%B'))
  months <- paste(as.character(months), collapse=", ")
  sdate_min <- format(min(as.Date(data_cube$attrs$Dates), na.rm = TRUE),
                      format = date_format)
  sdate_max <- format(max(as.Date(data_cube$attrs$Dates), na.rm = TRUE),
                      format = date_format)
  # Log the summary
  print(paste(Sys.time(), "DATA SUMMARY:"))
  
  print(paste(Sys.time(),object_name, "months:", months))
  print(paste(Sys.time(), object_name, "range:", sdate_min, "to", sdate_max))
  print(paste(Sys.time(), object_name, "dimensions:"))
  
  # Use capture.output() and for loop to display results neatly
  output_string <- capture.output(dim(data_cube$data))
  for (i in output_string) {
    print(paste(Sys.time(), i))
  }
  
  # Print statistical summary of the data for every variable
  print(paste(Sys.time(),"Statistical summary of the data in ", object_name, ":"))
  
  for (var_index in 1:data_cube$dims[['var']]) {
    variable_name <- data_cube$attrs$Variable$varName[var_index]
    variable_units <- data_cube$attrs$Variable$metadata[[variable_name]]$units
    print(paste(Sys.time(), "Variable: ", variable_name,
                " (units: ", variable_units, ")"))
    output_string <- capture.output(summary(Subset(data_cube$data,
                                                   along = 'var',
                                                   indices = list(var_index))))
    for (i in output_string) {
      print(paste(Sys.time(), i))
    }
  }
  
  # Number of NAs per time dimension and latitude/longitude dimensions
  
  list_na <- lapply(seq_along(dim(data_cube$data)), function(dim) {
    apply(data_cube$data, dim, function(x) sum(is.na(x)))
  })
  
  # Identify dimensions with NAs
  na_list <- sapply(list_na, function(x) which(x != 0) %>% paste(collapse = ",")) %>% unlist()
  names(na_list) <- names(dim(data_cube$data))
  
  # Count the number of NAs per identified dimension
  num_nas <- sapply(list_na, function(x) x[which(x != 0)] %>% paste(collapse = ",")) %>% unlist()
  names(num_nas) <- names(dim(data_cube$data))
  
  # Generate output strings
  output_na_list <- paste(names(na_list), na_list, sep = ": ", collapse = " ")
  output_num_nas <- paste(names(num_nas), num_nas, sep = ": ", collapse = " ")
  
  print(paste(Sys.time(), "Dimensions with NA values"))
  print(paste(Sys.time(), output_na_list))
  
  print(paste(Sys.time(), "Number of NAs per dimension")) 
  print(paste(Sys.time(), output_num_nas)) 
  
  print(paste(Sys.time(), "---------------------------------------------"))
  invisible(gc())
}

